@@ -5,13 +5,14 @@
 #include <stdbool.h>
 #include <wayland-egl.h>
 #include <EGL/egl.h>
+#include <EGL/eglext.h>
 #include <GLES2/gl2.h>
 #include <GLES2/gl2ext.h>
 
 struct ctx;
 
 #define MAX_PLANES 4
-typedef struct {
+typedef struct dmabuf {
     uint32_t width;
     uint32_t height;
     uint32_t drm_format;
@@ -23,6 +24,17 @@ typedef struct {
     uint64_t modifier;
 } dmabuf_t;
 
+typedef struct {
+    uint32_t drm_format;
+    size_t num_modifiers;
+    uint64_t * modifiers;
+} dmabuf_format_t;
+
+typedef struct {
+    size_t num_formats;
+    dmabuf_format_t * formats;
+} dmabuf_formats_t;
+
 typedef struct ctx_egl {
     EGLDisplay display;
     EGLContext context;
@@ -32,6 +44,11 @@ typedef struct ctx_egl {
 
     // extension functions
     PFNGLEGLIMAGETARGETTEXTURE2DOESPROC glEGLImageTargetTexture2DOES;
+    PFNEGLQUERYDMABUFFORMATSEXTPROC eglQueryDmaBufFormatsEXT;
+    PFNEGLQUERYDMABUFMODIFIERSEXTPROC eglQueryDmaBufModifiersEXT;
+
+    // supported dmabuf formats
+    dmabuf_formats_t dmabuf_formats;
 
     // texture size
     uint32_t width;
@@ -54,13 +71,14 @@ typedef struct ctx_egl {
 } ctx_egl_t;
 
 void wlm_egl_init(struct ctx * ctx);
+bool wlm_egl_query_dmabuf_formats(struct ctx * ctx);
+bool wlm_egl_check_errors(struct ctx * ctx, const char * msg);
 
 void wlm_egl_draw_texture(struct ctx * ctx);
 void wlm_egl_resize_viewport(struct ctx * ctx);
 void wlm_egl_resize_window(struct ctx * ctx);
 void wlm_egl_update_uniforms(struct ctx * ctx);
 void wlm_egl_freeze_framebuffer(struct ctx * ctx);
-bool wlm_egl_dmabuf_to_texture(struct ctx * ctx, dmabuf_t * dmabuf);
 
 void wlm_egl_cleanup(struct ctx * ctx);
 
