@@ -10,10 +10,14 @@ struct ctx;
 typedef struct mirror_backend {
     void (*do_capture)(struct ctx * ctx);
     void (*do_cleanup)(struct ctx * ctx);
+    void (*on_options_updated)(struct ctx * ctx);
     size_t fail_count;
 } mirror_backend_t;
 
-void wlm_mirror_dmabuf_init(struct ctx * ctx);
-void wlm_mirror_screencopy_init(struct ctx * ctx);
+void wlm_mirror_export_dmabuf_init(struct ctx * ctx);
+void wlm_mirror_screencopy_shm_init(struct ctx * ctx);
+void wlm_mirror_screencopy_dmabuf_init(struct ctx * ctx);
+void wlm_mirror_extcopy_shm_init(struct ctx * ctx);
+void wlm_mirror_extcopy_dmabuf_init(struct ctx * ctx);
 
 #endif
