@@ -12,6 +12,12 @@
 struct ctx;
 struct output_list_node;
 
+typedef struct fallback_backend fallback_backend_t;
+struct fallback_backend {
+    char * name;
+    void (*init)(struct ctx * ctx);
+};
+
 typedef struct ctx_mirror {
     struct output_list_node * current_target;
     struct wl_callback * frame_callback;
@@ -20,6 +26,7 @@ typedef struct ctx_mirror {
 
     // backend data
     mirror_backend_t * backend;
+    fallback_backend_t * fallback_backends;
     size_t auto_backend_index;
 
     // state flags
@@ -31,6 +38,7 @@ void wlm_mirror_backend_init(struct ctx * ctx);
 
 void wlm_mirror_output_removed(struct ctx * ctx, struct output_list_node * node);
 void wlm_mirror_update_title(struct ctx * ctx);
+void wlm_mirror_options_updated(struct ctx * ctx);
 
 void wlm_mirror_backend_fail(struct ctx * ctx);
 void wlm_mirror_cleanup(struct ctx * ctx);
