@@ -53,6 +53,11 @@ void wlm_egl_init(ctx_t * ctx) {
     ctx->egl.window = EGL_NO_SURFACE;
 
     ctx->egl.glEGLImageTargetTexture2DOES = NULL;
+    ctx->egl.eglQueryDmaBufFormatsEXT = NULL;
+    ctx->egl.eglQueryDmaBufModifiersEXT = NULL;
+
+    ctx->egl.dmabuf_formats.num_formats = 0;
+    ctx->egl.dmabuf_formats.formats = NULL;
 
     ctx->egl.width = 1;
     ctx->egl.height = 1;
@@ -150,6 +155,11 @@ void wlm_egl_init(ctx_t * ctx) {
         wlm_exit_fail(ctx);
     }
 
+    // query dmabuf formats
+    if (!wlm_egl_query_dmabuf_formats(ctx)) {
+        wlm_log_warn("egl::init(): can't list dmabuf modifiers, this might affect some dmabuf backends\n");
+    }
+
     // create vertex buffer object
     glGenBuffers(1, &ctx->egl.vbo);
     glBindBuffer(GL_ARRAY_BUFFER, ctx->egl.vbo);
@@ -261,6 +271,127 @@ void wlm_egl_init(ctx_t * ctx) {
     }
 }
 
+// --- query_dmabuf_formats ---
+
+bool wlm_egl_query_dmabuf_formats(ctx_t * ctx) {
+    //if (!has_extension("EGL_EXT_image_dma_buf_import_modifiers")) {
+    //    wlm_log_error("egl::init(): missing EGL extension EGL_EXT_image_dma_buf_import_modifiers\n");
+    //    return false;
+    //}
+
+    // get pointers to optional functions provided by extensions
+    // - eglQueryDmaBufFormatsEXT: for getting dmabuf formats
+    ctx->egl.eglQueryDmaBufFormatsEXT = (PFNEGLQUERYDMABUFFORMATSEXTPROC)eglGetProcAddress("eglQueryDmaBufFormatsEXT");
+    if (ctx->egl.eglQueryDmaBufFormatsEXT == NULL) {
+        wlm_log_error("egl::init(): failed to get pointer to eglQueryDmaBufFormatsEXT\n");
+        return false;
+    }
+
+    // - eglQueryDmaBufModifiersEXT: for getting dmabuf format modifiers
+    ctx->egl.eglQueryDmaBufModifiersEXT = (PFNEGLQUERYDMABUFMODIFIERSEXTPROC)eglGetProcAddress("eglQueryDmaBufModifiersEXT");
+    if (ctx->egl.eglQueryDmaBufModifiersEXT == NULL) {
+        wlm_log_error("egl::init(): failed to get pointer to eglQueryDmaBufModifiersEXT\n");
+        return false;
+    }
+
+    EGLint num_formats = 0;
+    if (!ctx->egl.eglQueryDmaBufFormatsEXT(ctx->egl.display, 0, NULL, &num_formats)) {
+        wlm_log_error("egl::init(): failed to query number of egl dmabuf formats\n");
+        return false;
+    }
+
+    EGLint * egl_drm_formats = calloc(num_formats, sizeof *egl_drm_formats);
+    if (egl_drm_formats == NULL) {
+        wlm_log_error("egl::init(): failed to allocate egl dmabuf format array\n");
+        return false;
+    }
+
+    if (!ctx->egl.eglQueryDmaBufFormatsEXT(ctx->egl.display, num_formats, egl_drm_formats, &num_formats)) {
+        free(egl_drm_formats);
+        wlm_log_error("egl::init(): failed to query egl dmabuf formats\n");
+        return false;
+    }
+
+    dmabuf_format_t * dmabuf_formats = calloc(num_formats, sizeof *dmabuf_formats);
+    if (dmabuf_formats == NULL) {
+        free(egl_drm_formats);
+        wlm_log_error("egl::init(): failed to allocate dmabuf format array\n");
+        return false;
+    }
+
+    bool success = true;
+    for (size_t i = 0; i < (size_t)num_formats; i++) {
+        EGLint num_modifiers = 0;
+        if (!ctx->egl.eglQueryDmaBufModifiersEXT(ctx->egl.display, egl_drm_formats[i], 0, NULL, NULL, &num_modifiers)) {
+            wlm_log_error("egl::init(): failed to allocate dmabuf format array\n");
+            success = false;
+            break;
+        }
+
+        EGLuint64KHR * egl_drm_modifiers = calloc(num_modifiers, sizeof *egl_drm_modifiers);
+        if (egl_drm_modifiers == NULL) {
+            wlm_log_error("egl::init(): failed to allocate egl dmabuf format modifier array for format %x\n", egl_drm_formats[i]);
+            success = false;
+            break;
+        }
+
+        if (!ctx->egl.eglQueryDmaBufModifiersEXT(ctx->egl.display, egl_drm_formats[i], 0, (EGLuint64KHR*)egl_drm_modifiers, NULL, &num_modifiers)) {
+            free(egl_drm_modifiers);
+            wlm_log_error("egl::init(): failed query egl dmabuf format modifiers for format %x\n", egl_drm_formats[i]);
+            success = false;
+            break;
+        }
+
+        uint64_t * modifiers = calloc(num_modifiers, sizeof *modifiers);
+        if (modifiers == NULL) {
+            free(egl_drm_modifiers);
+            wlm_log_error("egl::init(): failed to allocate modifier array for format %x\n", egl_drm_formats[i]);
+            success = false;
+            break;
+        }
+
+        for (size_t j = 0; j < (size_t)num_modifiers; j++) {
+            modifiers[j] = egl_drm_modifiers[j];
+        }
+
+        dmabuf_formats[i].drm_format = egl_drm_formats[i];
+        dmabuf_formats[i].num_modifiers = num_modifiers;
+        dmabuf_formats[i].modifiers = modifiers;
+
+        free(egl_drm_modifiers);
+    }
+
+    free(egl_drm_formats);
+
+    // cleanup
+    if (!success) {
+        for (size_t i = 0; i < (size_t)num_formats; i++) {
+            if (dmabuf_formats[i].modifiers != NULL) {
+                free(dmabuf_formats[i].modifiers);
+            }
+        }
+        free(dmabuf_formats);
+        return false;
+    }
+
+    ctx->egl.dmabuf_formats.num_formats = num_formats;
+    ctx->egl.dmabuf_formats.formats = dmabuf_formats;
+    return true;
+}
+
+bool wlm_egl_check_errors(struct ctx * ctx, const char * msg) {
+    GLenum error = glGetError();
+    if (error == GL_NO_ERROR) return true;
+
+    while (error != GL_NO_ERROR) {
+        wlm_log_error("egl::check_errors(): %s: GL error 0x%x\n", msg, error);
+        error = glGetError();
+    }
+    return false;
+
+    (void)ctx;
+}
+
 // --- draw_texture ---
 
 void wlm_egl_draw_texture(ctx_t *ctx) {
@@ -439,106 +570,13 @@ void wlm_egl_update_uniforms(ctx_t * ctx) {
 void wlm_egl_freeze_framebuffer(struct ctx * ctx) {
     glBindFramebuffer(GL_FRAMEBUFFER, ctx->egl.freeze_framebuffer);
     glBindTexture(GL_TEXTURE_2D, ctx->egl.freeze_texture);
+
     glCopyTexImage2D(GL_TEXTURE_2D, 0, ctx->egl.format, 0, 0, ctx->egl.width, ctx->egl.height, 0);
+    wlm_egl_check_errors(ctx, "failed to copy frame to freeze texture");
+
     glBindFramebuffer(GL_FRAMEBUFFER, 0);
 }
 
-// --- dmabuf_to_texture ---
-
-static const EGLAttrib fd_attribs[] = {
-    EGL_DMA_BUF_PLANE0_FD_EXT,
-    EGL_DMA_BUF_PLANE1_FD_EXT,
-    EGL_DMA_BUF_PLANE2_FD_EXT,
-    EGL_DMA_BUF_PLANE3_FD_EXT
-};
-_Static_assert(ARRAY_LENGTH(fd_attribs) == MAX_PLANES, "fd_attribs has incorrect length");
-
-static const EGLAttrib offset_attribs[] = {
-    EGL_DMA_BUF_PLANE0_OFFSET_EXT,
-    EGL_DMA_BUF_PLANE1_OFFSET_EXT,
-    EGL_DMA_BUF_PLANE2_OFFSET_EXT,
-    EGL_DMA_BUF_PLANE3_OFFSET_EXT
-};
-_Static_assert(ARRAY_LENGTH(offset_attribs) == MAX_PLANES, "offset_attribs has incorrect length");
-
-static const EGLAttrib stride_attribs[] = {
-    EGL_DMA_BUF_PLANE0_PITCH_EXT,
-    EGL_DMA_BUF_PLANE1_PITCH_EXT,
-    EGL_DMA_BUF_PLANE2_PITCH_EXT,
-    EGL_DMA_BUF_PLANE3_PITCH_EXT
-};
-_Static_assert(ARRAY_LENGTH(stride_attribs) == MAX_PLANES, "stride_attribs has incorrect length");
-
-static const EGLAttrib modifier_low_attribs[] = {
-    EGL_DMA_BUF_PLANE0_MODIFIER_LO_EXT,
-    EGL_DMA_BUF_PLANE1_MODIFIER_LO_EXT,
-    EGL_DMA_BUF_PLANE2_MODIFIER_LO_EXT,
-    EGL_DMA_BUF_PLANE3_MODIFIER_LO_EXT
-};
-_Static_assert(ARRAY_LENGTH(modifier_low_attribs) == MAX_PLANES, "modifier_low_attribs has incorrect length");
-
-static const EGLAttrib modifier_high_attribs[] = {
-    EGL_DMA_BUF_PLANE0_MODIFIER_HI_EXT,
-    EGL_DMA_BUF_PLANE1_MODIFIER_HI_EXT,
-    EGL_DMA_BUF_PLANE2_MODIFIER_HI_EXT,
-    EGL_DMA_BUF_PLANE3_MODIFIER_HI_EXT
-};
-_Static_assert(ARRAY_LENGTH(modifier_high_attribs) == MAX_PLANES, "modifier_high_attribs has incorrect length");
-
-bool wlm_egl_dmabuf_to_texture(ctx_t * ctx, dmabuf_t * dmabuf) {
-    if (dmabuf->planes > MAX_PLANES) {
-        wlm_log_error("egl::dmabuf_to_texture(): too many planes, got %zd, can support at most %d\n", dmabuf->planes, MAX_PLANES);
-        return false;
-    }
-
-    int i = 0;
-    EGLAttrib * image_attribs = malloc((6 + 10 * dmabuf->planes + 1) * sizeof (EGLAttrib));
-    if (image_attribs == NULL) {
-        wlm_log_error("egl::dmabuf_to_texture(): failed to allocate EGL image attribs\n");
-        return false;
-    }
-
-    image_attribs[i++] = EGL_WIDTH;
-    image_attribs[i++] = dmabuf->width;
-    image_attribs[i++] = EGL_HEIGHT;
-    image_attribs[i++] = dmabuf->height;
-    image_attribs[i++] = EGL_LINUX_DRM_FOURCC_EXT;
-    image_attribs[i++] = dmabuf->drm_format;
-
-    for (size_t j = 0; j < dmabuf->planes; j++) {
-        image_attribs[i++] = fd_attribs[j];
-        image_attribs[i++] = dmabuf->fds[j];
-        image_attribs[i++] = offset_attribs[j];
-        image_attribs[i++] = dmabuf->offsets[j];
-        image_attribs[i++] = stride_attribs[j];
-        image_attribs[i++] = dmabuf->strides[j];
-        image_attribs[i++] = modifier_low_attribs[j];
-        image_attribs[i++] = (uint32_t)dmabuf->modifier;
-        image_attribs[i++] = modifier_high_attribs[j];
-        image_attribs[i++] = (uint32_t)(dmabuf->modifier >> 32);
-    }
-
-    image_attribs[i++] = EGL_NONE;
-
-    // create EGLImage from dmabuf with attribute array
-    EGLImage frame_image = eglCreateImage(ctx->egl.display, EGL_NO_CONTEXT, EGL_LINUX_DMA_BUF_EXT, NULL, image_attribs);
-    free(image_attribs);
-
-    if (frame_image == EGL_NO_IMAGE) {
-        wlm_log_error("egl::dmabuf_to_texture(): failed to create EGL image from dmabuf: error = %x\n", eglGetError());
-        return false;
-    }
-
-    // convert EGLImage to GL texture
-    glBindTexture(GL_TEXTURE_2D, ctx->egl.texture);
-    ctx->egl.glEGLImageTargetTexture2DOES(GL_TEXTURE_2D, frame_image);
-
-    // destroy temporary image
-    eglDestroyImage(ctx->egl.display, frame_image);
-
-    return true;
-}
-
 // --- cleanup_egl ---
 
 void wlm_egl_cleanup(ctx_t *ctx) {
@@ -546,6 +584,15 @@ void wlm_egl_cleanup(ctx_t *ctx) {
 
     wlm_log_debug(ctx, "egl::cleanup(): destroying EGL objects\n");
 
+    if (ctx->egl.dmabuf_formats.formats != NULL) {
+        for (size_t i = 0; i < ctx->egl.dmabuf_formats.num_formats; i++) {
+            if (ctx->egl.dmabuf_formats.formats[i].modifiers != NULL) {
+                free(ctx->egl.dmabuf_formats.formats[i].modifiers);
+            }
+        }
+        free(ctx->egl.dmabuf_formats.formats);
+    }
+
     if (ctx->egl.shader_program != 0) glDeleteProgram(ctx->egl.shader_program);
     if (ctx->egl.freeze_framebuffer != 0) glDeleteFramebuffers(1, &ctx->egl.freeze_framebuffer);
     if (ctx->egl.freeze_texture != 0) glDeleteTextures(1, &ctx->egl.freeze_texture);
