@@ -23,8 +23,9 @@ void wlm_opt_init(ctx_t * ctx) {
 }
 
 void wlm_cleanup_opt(ctx_t * ctx) {
-    if (ctx->opt.output != NULL) free(ctx->opt.output);
-    if (ctx->opt.fullscreen_output != NULL) free(ctx ->opt.fullscreen_output);
+    free(ctx->opt.output);
+    free(ctx->opt.fullscreen_output);
+    free(ctx->opt.window_title);
 }
 
 bool wlm_opt_parse_scaling(scale_t * scaling, scale_filter_t * scaling_filter, const char * scaling_arg) {
@@ -52,11 +53,30 @@ bool wlm_opt_parse_backend(backend_t * backend, const char * backend_arg) {
     if (strcmp(backend_arg, "auto") == 0) {
         *backend = BACKEND_AUTO;
         return true;
+    } else if (strcmp(backend_arg, "export-dmabuf") == 0) {
+        *backend = BACKEND_EXPORT_DMABUF;
+        return true;
     } else if (strcmp(backend_arg, "dmabuf") == 0) {
-        *backend = BACKEND_DMABUF;
+        wlm_log_warn("options::parse_backend(): the name 'dmabuf' is deprecated, use 'export-dmabuf' for the wlr-export-dmabuf backend instead\n");
+        *backend = BACKEND_EXPORT_DMABUF;
         return true;
     } else if (strcmp(backend_arg, "screencopy") == 0) {
-        *backend = BACKEND_SCREENCOPY;
+        *backend = BACKEND_SCREENCOPY_AUTO;
+        return true;
+    } else if (strcmp(backend_arg, "screencopy-shm") == 0) {
+        *backend = BACKEND_SCREENCOPY_SHM;
+        return true;
+    } else if (strcmp(backend_arg, "screencopy-dmabuf") == 0) {
+        *backend = BACKEND_SCREENCOPY_DMABUF;
+        return true;
+    } else if (strcmp(backend_arg, "extcopy") == 0) {
+        *backend = BACKEND_EXTCOPY_AUTO;
+        return true;
+    } else if (strcmp(backend_arg, "extcopy-shm") == 0) {
+        *backend = BACKEND_EXTCOPY_SHM;
+        return true;
+    } else if (strcmp(backend_arg, "extcopy-dmabuf") == 0) {
+        *backend = BACKEND_EXTCOPY_DMABUF;
         return true;
     } else {
         return false;
@@ -352,10 +372,10 @@ void wlm_opt_usage(ctx_t * ctx) {
     printf("  -f,   --freeze                freeze the current image on the screen\n");
     printf("        --unfreeze              resume the screen capture after a freeze\n");
     printf("        --toggle-freeze         toggle freeze state of screen capture\n");
-    printf("  -F,   --fullscreen            open wl-mirror as fullscreen\n");
-    printf("        --no-fullscreen         open wl-mirror as a window (default)\n");
-    printf("        --fullscreen-output O   open wl-mirror as fullscreen on output O\n");
-    printf("        --no-fullscreen-output  open wl-mirror as fullscreen on the current output (default)\n");
+    printf("  -F,   --fullscreen            display wl-mirror as fullscreen\n");
+    printf("        --no-fullscreen         display wl-mirror as a window (default)\n");
+    printf("        --fullscreen-output O   set fullscreen target output to output O, implies --fullscreen\n");
+    printf("        --no-fullscreen-output  unset fullscreen target output, implies --no-fullscreen (default)\n");
     printf("  -s f, --scaling fit           scale to fit (default)\n");
     printf("  -s c, --scaling cover         scale to cover, cropping if needed\n");
     printf("  -s e, --scaling exact         only scale to exact multiples of the output size\n");
@@ -369,9 +389,14 @@ void wlm_opt_usage(ctx_t * ctx) {
     printf("        --title N               specify a custom title N for the mirror window\n");
     printf("\n");
     printf("backends:\n");
-    printf("  - auto        automatically try the backends in order and use the first that works (default)\n");
-    printf("  - dmabuf      use the wlr-export-dmabuf-unstable-v1 protocol to capture outputs\n");
-    printf("  - screencopy  use the wlr-screencopy-unstable-v1 protocol to capture outputs\n");
+    printf("  - auto                automatically try the backends in order of efficiency and use the first that works (default)\n");
+    printf("  - export-dmabuf       use the wlr-export-dmabuf-unstable-v1 protocol to capture outputs\n");
+    printf("  - screencopy          use the wlr-screencopy-unstable-v1 protocol to capture outputs (auto)\n");
+    printf("  - screencopy-dmabuf   use the wlr-screencopy-unstable-v1 protocol to capture outputs (via DMA-BUF)\n");
+    printf("  - screencopy-shm      use the wlr-screencopy-unstable-v1 protocol to capture outputs (via SHM)\n");
+    printf("  - extcopy             use the ext-image-copy-capture-v1 protocol to capture outputs (auto)\n");
+    printf("  - extcopy-dmabuf      use the ext-image-copy-capture-v1 protocol to capture outputs (via DMA-BUF)\n");
+    printf("  - extcopy-shm         use the ext-image-copy-capture-v1 protocol to capture outputs (via SHM)\n");
     printf("\n");
     printf("transforms:\n");
     printf("  transforms are specified as a dash-separated list of flips followed by a rotation\n");
@@ -405,9 +430,9 @@ void wlm_opt_usage(ctx_t * ctx) {
     printf("  - {target_width}, {target_height}\n");
     printf("    {target_output}:                 info about the mirrored device\n");
     printf("  a few perhaps useful examples:\n");
-    printf("    --title='Wayland Mirror Output {target_output}'\n");
-    printf("    --title='{target_output}:{width}x{height}+{x}+{y}'\n");
-    printf("    --title='resize set {width} {height} move position {x} {y}'\n");
+    printf("    --title 'Wayland Mirror Output {target_output}'\n");
+    printf("    --title '{target_output}:{width}x{height}+{x}+{y}'\n");
+    printf("    --title 'resize set {width} {height} move position {x} {y}'\n");
     wlm_cleanup(ctx);
     exit(0);
 }
@@ -473,6 +498,7 @@ void wlm_opt_parse(ctx_t * ctx, int argc, char ** argv) {
             }
         } else if (strcmp(argv[0], "--no-fullscreen-output") == 0) {
             free(ctx->opt.fullscreen_output);
+            ctx->opt.fullscreen = false;
             ctx->opt.fullscreen_output = NULL;
             new_fullscreen_output = true;
         } else if (strcmp(argv[0], "-s") == 0 || strcmp(argv[0], "--scaling") == 0) {
@@ -548,7 +574,8 @@ void wlm_opt_parse(ctx_t * ctx, int argc, char ** argv) {
                     wlm_log_error("options::parse(): invalid empty title\n");
                     if (is_cli_args) wlm_exit_fail(ctx);
                 } else {
-                    ctx->opt.window_title = argv[1];
+                    free(ctx->opt.window_title);
+                    ctx->opt.window_title = strdup(argv[1]);
                 }
 
                 argv++;
@@ -618,7 +645,7 @@ void wlm_opt_parse(ctx_t * ctx, int argc, char ** argv) {
         ctx->opt.output != NULL && ctx->opt.fullscreen_output != NULL &&
         strcmp(ctx->opt.output, ctx->opt.fullscreen_output) == 0
     ) {
-        wlm_log_error("options::parse(): fullscreen_output cannot be same as the output to be mirrored");
+        wlm_log_error("options::parse(): fullscreen_output cannot be same as the output to be mirrored\n");
         wlm_exit_fail(ctx);
     }
 
@@ -651,6 +678,7 @@ void wlm_opt_parse(ctx_t * ctx, int argc, char ** argv) {
     if (!is_cli_args) {
         wlm_egl_update_uniforms(ctx);
         wlm_mirror_update_title(ctx);
+        wlm_mirror_options_updated(ctx);
     }
 }
 
