@@ -1,3 +1,4 @@
+#include "wlm/proto/ext-image-capture-source-v1.h"
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -270,7 +271,7 @@ static void on_registry_add(
         }
 
         // bind dmabuf manager object
-        // - for mirror-dmabuf backend
+        // - for mirror-export-dmabuf backend
         ctx->wl.dmabuf_manager = (struct zwlr_export_dmabuf_manager_v1 *)wl_registry_bind(
             registry, id, &zwlr_export_dmabuf_manager_v1_interface, 1
         );
@@ -287,6 +288,42 @@ static void on_registry_add(
             registry, id, &zwlr_screencopy_manager_v1_interface, 3
         );
         ctx->wl.screencopy_manager_id = id;
+    } else if (strcmp(interface, ext_image_copy_capture_manager_v1_interface.name) == 0) {
+        if (ctx->wl.copy_capture_manager != NULL) {
+            wlm_log_error("wayland::on_registry_add(): duplicate screencopy_manager\n");
+            wlm_exit_fail(ctx);
+        }
+
+        // bind image copy capture manager object
+        // - for extcopy backend
+        ctx->wl.copy_capture_manager = (struct ext_image_copy_capture_manager_v1 *)wl_registry_bind(
+            registry, id, &ext_image_copy_capture_manager_v1_interface, 1
+        );
+        ctx->wl.copy_capture_manager_id = id;
+    } else if (strcmp(interface, ext_output_image_capture_source_manager_v1_interface.name) == 0) {
+        if (ctx->wl.output_capture_source_manager != NULL) {
+            wlm_log_error("wayland::on_registry_add(): duplicate screencopy_manager\n");
+            wlm_exit_fail(ctx);
+        }
+
+        // bind output capture source manager object
+        // - for extcopy backend
+        ctx->wl.output_capture_source_manager = (struct ext_output_image_capture_source_manager_v1 *)wl_registry_bind(
+            registry, id, &ext_output_image_capture_source_manager_v1_interface, 1
+        );
+        ctx->wl.output_capture_source_manager_id = id;
+    } else if (strcmp(interface, ext_foreign_toplevel_image_capture_source_manager_v1_interface.name) == 0) {
+        if (ctx->wl.toplevel_capture_source_manager != NULL) {
+            wlm_log_error("wayland::on_registry_add(): duplicate screencopy_manager\n");
+            wlm_exit_fail(ctx);
+        }
+
+        // bind toplevel capture source manager object
+        // - for extcopy backend
+        ctx->wl.toplevel_capture_source_manager = (struct ext_foreign_toplevel_image_capture_source_manager_v1 *)wl_registry_bind(
+            registry, id, &ext_foreign_toplevel_image_capture_source_manager_v1_interface, 1
+        );
+        ctx->wl.toplevel_capture_source_manager_id = id;
     } else if (strcmp(interface, wl_shm_interface.name) == 0) {
         if (ctx->wl.shm != NULL) {
             wlm_log_error("wayland::on_registry_add(): duplicate shm\n");
@@ -299,6 +336,18 @@ static void on_registry_add(
             registry, id, &wl_shm_interface, 1
         );
         ctx->wl.shm_id = id;
+    } else if (strcmp(interface, zwp_linux_dmabuf_v1_interface.name) == 0) {
+        if (ctx->wl.linux_dmabuf != NULL) {
+            wlm_log_error("wayland::on_registry_add(): duplicate linux_dmabuf\n");
+            wlm_exit_fail(ctx);
+        }
+
+        // bind linux_dmabuf object
+        // - for mirror-screencopy backend
+        ctx->wl.linux_dmabuf = (struct zwp_linux_dmabuf_v1 *)wl_registry_bind(
+            registry, id, &zwp_linux_dmabuf_v1_interface, 4
+        );
+        ctx->wl.linux_dmabuf_id = id;
     } else if (strcmp(interface, wl_output_interface.name) == 0) {
         // allocate output node
         output_list_node_t * node = malloc(sizeof (output_list_node_t));
@@ -403,9 +452,24 @@ static void on_registry_remove(
     } else if (id == ctx->wl.output_manager_id) {
         wlm_log_error("wayland::on_registry_remove(): output_manager disappeared\n");
         wlm_exit_fail(ctx);
+    } else if (id == ctx->wl.shm_id) {
+        wlm_log_error("wayland::on_registry_remove(): shm disappeared\n");
+        wlm_exit_fail(ctx);
+    } else if (id == ctx->wl.linux_dmabuf_id) {
+        wlm_log_error("wayland::on_registry_remove(): linux_dmabuf disappeared\n");
+        wlm_exit_fail(ctx);
     } else if (id == ctx->wl.dmabuf_manager_id) {
         wlm_log_error("wayland::on_registry_remove(): dmabuf_manager disappeared\n");
         wlm_exit_fail(ctx);
+    } else if (id == ctx->wl.copy_capture_manager_id) {
+        wlm_log_error("wayland::on_registry_remove(): copy_capture_manager disappeared\n");
+        wlm_exit_fail(ctx);
+    } else if (id == ctx->wl.output_capture_source_manager_id) {
+        wlm_log_error("wayland::on_registry_remove(): output_capture_source_manager disappeared\n");
+        wlm_exit_fail(ctx);
+    } else if (id == ctx->wl.toplevel_capture_source_manager_id) {
+        wlm_log_error("wayland::on_registry_remove(): toplevel_capture_source_manager disappeared\n");
+        wlm_exit_fail(ctx);
     } else {
         {
             output_list_node_t ** link = &ctx->wl.outputs;
@@ -525,14 +589,15 @@ static void on_surface_enter(
     bool first_output = ctx->wl.current_output == NULL;
     ctx->wl.current_output = node;
 
-    // set window fullscreen now if no specific output requested
-    if (first_output && ctx->opt.fullscreen) {
-        wlm_wayland_window_set_fullscreen(ctx);
-    }
-
     wlm_log_debug(ctx, "wayland::on_surface_enter(): updating window scale\n");
     wlm_wayland_window_update_scale(ctx, node->scale, false);
 
+    // set fullscreen on current output if requested by initial options
+    if (first_output && ctx->opt.fullscreen && ctx->opt.fullscreen_output == NULL) {
+        wlm_log_debug(ctx, "wayland::on_surface_enter(): fullscreening on current output\n");
+        wlm_wayland_window_set_fullscreen(ctx);
+    }
+
     (void)surface;
 }
 
@@ -844,13 +909,23 @@ void wlm_wayland_init(ctx_t * ctx) {
     ctx->wl.output_manager = NULL;
     ctx->wl.output_manager_id = 0;
 
-    ctx->wl.dmabuf_manager = NULL;
-    ctx->wl.dmabuf_manager_id = 0;
     ctx->wl.shm = NULL;
     ctx->wl.shm_id = 0;
+    ctx->wl.linux_dmabuf = NULL;
+    ctx->wl.linux_dmabuf_id = 0;
+
+    ctx->wl.dmabuf_manager = NULL;
+    ctx->wl.dmabuf_manager_id = 0;
     ctx->wl.screencopy_manager = NULL;
     ctx->wl.screencopy_manager_id = 0;
 
+    ctx->wl.copy_capture_manager = NULL;
+    ctx->wl.output_capture_source_manager = NULL;
+    ctx->wl.toplevel_capture_source_manager = NULL;
+    ctx->wl.copy_capture_manager_id = 0;
+    ctx->wl.output_capture_source_manager_id = 0;
+    ctx->wl.toplevel_capture_source_manager_id = 0;
+
     ctx->wl.outputs = NULL;
     ctx->wl.seats = NULL;
 
@@ -886,6 +961,9 @@ void wlm_wayland_init(ctx_t * ctx) {
     ctx->wl.closing = false;
     ctx->wl.initialized = true;
 
+    wlm_wayland_shm_init(ctx);
+    wlm_wayland_dmabuf_init(ctx);
+
     // connect to display
     ctx->wl.display = wl_display_connect(NULL);
     if (ctx->wl.display == NULL) {
@@ -1021,16 +1099,18 @@ void wlm_wayland_init(ctx_t * ctx) {
     wl_display_roundtrip(ctx->wl.display);
 #endif
 
-    // set fullscreen on xdg_toplevel
-    if (ctx->opt.fullscreen && ctx->opt.fullscreen_output != NULL) {
-        wlm_wayland_window_set_fullscreen(ctx);
-    }
     // check if surface is configured
     // - expecting surface to be configured at this point
     if (!ctx->wl.configured) {
         wlm_log_error("wayland::init(): surface not configured\n");
         wlm_exit_fail(ctx);
     }
+
+    // set fullscreen on target output if requested by initial options
+    if (ctx->opt.fullscreen && ctx->opt.fullscreen_output != NULL) {
+        wlm_log_debug(ctx, "wayland::init(): fullscreening on target output\n");
+        wlm_wayland_window_set_fullscreen(ctx);
+    }
 }
 
 // --- close_window ---
@@ -1102,6 +1182,9 @@ void wlm_wayland_cleanup(ctx_t *ctx) {
 
     wlm_log_debug(ctx, "wayland::cleanup(): destroying wayland objects\n");
 
+    wlm_wayland_shm_cleanup(ctx);
+    wlm_wayland_dmabuf_cleanup(ctx);
+
     // deregister event handler
     wlm_event_remove_fd(ctx, &ctx->wl.event_handler);
 
@@ -1137,8 +1220,12 @@ void wlm_wayland_cleanup(ctx_t *ctx) {
         ctx->wl.seats = NULL;
     }
 
+    if (ctx->wl.copy_capture_manager != NULL) ext_image_copy_capture_manager_v1_destroy(ctx->wl.copy_capture_manager);
+    if (ctx->wl.output_capture_source_manager != NULL) ext_output_image_capture_source_manager_v1_destroy(ctx->wl.output_capture_source_manager);
+    if (ctx->wl.toplevel_capture_source_manager != NULL) ext_foreign_toplevel_image_capture_source_manager_v1_destroy(ctx->wl.toplevel_capture_source_manager);
     if (ctx->wl.dmabuf_manager != NULL) zwlr_export_dmabuf_manager_v1_destroy(ctx->wl.dmabuf_manager);
     if (ctx->wl.screencopy_manager != NULL) zwlr_screencopy_manager_v1_destroy(ctx->wl.screencopy_manager);
+    if (ctx->wl.linux_dmabuf != NULL) zwp_linux_dmabuf_v1_destroy(ctx->wl.linux_dmabuf);
     if (ctx->wl.shm != NULL) wl_shm_destroy(ctx->wl.shm);
 #ifdef WITH_LIBDECOR
     if (ctx->wl.libdecor_frame != NULL) libdecor_frame_unref(ctx->wl.libdecor_frame);
